/*
  Program:      Mine Mayhem
  File:         times.cc
  Date: 	July, 1996 to February, 1997, version 1.00.
		26 October to 3 December,     version 2.00.

  Everything to do with record times.

  Freeware, Copyright 1996-7 Jason Hood.

  You are free to use this code, or a portion thereof, as long as an
  appropriate acknowledgement is made.

  v2.00: added Table::str() and Table::Entry::str() functions;
	 added highlight when a new entry is added;
	 added "deviation" option;
	 used "---.---" for "Still waiting" times;
	 modified the level checkbox radiobutton emulation;
	 better centring of the checkboxes.
*/

#include <time.h>
#include <stdio.h>
#include "times.h"
#include "minefld.h"

extern BYTE HighLightColour;


// Constructor for an individual entry in a times table, excluding the time
// itself.
Table::Entry::Entry() : start(0), finish(0), wrap(false)
{
  strcpy(name, "Still waiting");
  strcpy(date, "??-???-????");
}

// Convert the entry to a string.
// If clear is true, format for the clear game, otherwise the cross.
char* Table::Entry::str(bool clear, bool deviation, double avg) const
{
  static char buffer[80],
	      *strstart[]  = { "None", "One ", "Auto"         },
	      *strfinish[] = { "Both", "Open", "Mark", "Itha" },
	      *strwrap[]   = { "",     "Wrap"                 };

  if (*date == '?')
  {
    sprintf(buffer, "---.---  %-20s  %s", name, date);
  }
  else
  {
    double t = (deviation) ? ((time - avg) / avg) * 100.0 : time / 1000.0;
    if (clear)
	sprintf(buffer, "%7.3f  %-20s  %s  %s  %s  %s",
			t, name, date,
			strstart[start], strfinish[finish], strwrap[wrap]);
    else
	sprintf(buffer, "%7.3f  %-20s  %s  %4d  %4d",
			t, name, date, spaces, mines);
  }

  return buffer;
}


// Constructor for an entire times table.
Table::Table() : games(0), timetotal(0), spacetotal(0), minetotal(0)
{
  for (unsigned j = 0; j < Entries; j++)
  {
    fastest[j].time   = 999999; 	  // ie. 999.999
    fastest[j].spaces =
    fastest[j].mines  = 0;

    slowest[j].time   = 0;
    slowest[j].spaces =
    slowest[j].mines  = 999;
  }
}


/* -------------------------------   Enter   --------------------------------

  Update the number of games and total time for this table. If time is a
  record, then ask for the name and insert it at the appropriate position.
  Return -1 if the time was not a record, otherwise 0 - 4 for the fast times,
  and 5 - 9 for the slow.

*/

int Table::Enter(unsigned time, unsigned spaces, unsigned mines)
{
  int isRecord = -1;

  ++games;
  timetotal  += time;
  spacetotal += spaces;
  minetotal  += mines;

  for (unsigned j = 0; j < Entries; j++)
  {
    if (time <= fastest[j].time)        // Same time is regarded as better
    {                                   //  to reflect the later date
      shift(fastest, j);
      Stats(fastest, j, time, spaces, mines);
      isRecord = j;
      break;
    }
  }
  for (unsigned j = 0; j < Entries; j++)
  {
    if (time >= slowest[j].time)        // Same time is regarded as worse
    {                                   //  to reflect the later date
      shift(slowest, j);
      if (isRecord != -1) slowest[j] = fastest[isRecord];
      else
      {
	Stats(slowest, j, time, spaces, mines);
	isRecord = j + Entries;
      }
      break;
    }
  }
  return isRecord;
}

// Make room for the new entry.
void Table::shift(Entry which[], unsigned from)
{
  // which[from+1..Entries-1] = which[from..Entries-2]
  for (unsigned j = Entries-1; j > from; j--)
  {
    which[j] = which[j-1];
  }
}

// Place the values in the entry.
void Table::Stats(Entry which[], unsigned position,
		  unsigned time, unsigned spaces, unsigned mines)
{
  GetDate(which[position]);
  AskName(which, position);
  which[position].time	 = time;
  which[position].start  = Start;
  which[position].finish = Finish;
  which[position].wrap	 = (isWrap && field->canWrap());
  which[position].spaces = spaces;
  which[position].mines  = mines;
}

// Place the date in the entry as "dd-Mmm-yyyy" (eg. 15-Feb-1997).
void Table::GetDate(Entry& entry)
{
  time_t now;
  time(&now);
  strftime(entry.date, DateLen, "%e-%b-%Y", localtime(&now));
}


// Show the player's position in which table and ask for the name.
void Table::AskName(Entry which[], unsigned position)
{
  static char *place[] = { "", " second", " third", " fourth", " fifth" },
              *fast = "Wow! That is the%s fastest time!",
              *slow = "Unfortunately, that is the%s slowest time.";

  char buffer[80], *message;

  message = (which == fastest) ? fast : slow;
  sprintf(buffer, message, place[position]);
  strcpy(which[position].name, EnterNameDlg(buffer).Run());
}


// Format the statistics as a string.
char* Table::str(bool clear) const
{
  static char buffer[80];

  if (games == 0) *buffer = 0;
  else
  {
    double avg = ((double)timetotal / games) / 1000.0;
    if (clear)
    {
      sprintf(buffer, "%7.3f  Average               %d cleared", avg, games);
    }
    else
    {
      char temp[13];
      sprintf(temp, "%d crossed", games);
      sprintf(buffer, "%7.3f  Average               %-11s  %4d  %4d",
		      avg, temp, spacetotal/games, minetotal/games);
    }
  }
  return buffer;
}


/* -------------------------------   Times   -------------------------------- */

Times::Times()
{
  head = prevCustom = new CustomTimes;
  head->next = NULL;
}


Times::~Times()
{
  CustomTimes* list;
  while (head != NULL)
  {
    list = head;
    head = head->next;
    delete list;
  }
}


/* -----------------------------   FindTable   ------------------------------

  Given the level, type of game and automatic options, return a pointer to
  the appropriate table. If level is CUSTOM and there is no matching layout,
  create a new one.

*/

Table* Times::FindTable(int level, int game, bool automark, bool autoopen)
{
  Table* table;

  switch (level)
  {
    case BEGINNER:     table = Beginner;     break;
    case INTERMEDIATE: table = Intermediate; break;
    case EXPERT:       table = Expert;	     break;
    default: if (field->Custom == prevCustom->layout) table = prevCustom->times;
             else
             {
               CustomTimes* list = head;
               for (; list->next; list = list->next)
               {
		 if (list->layout == field->Custom)
                 {
		   table      = list->times;
                   prevCustom = list;
                   break;
                 }
               }
               if (list->next == NULL)
               {
		 list->layout = field->Custom;
		 table	      = list->times;
		 prevCustom   = list;
		 list	      =
		 list->next   = new CustomTimes;
		 list->next   = NULL;
		 field->CustomPopup.Insert(field->Custom);
               }
             }
             break;
  }
  //	       Clear   Cross
  // Normal:	 0	 3
  // AutoMark:	 2	 5
  // AutoOpen:	 1	 4
  return &table[(game == CROSS)*3 + automark*2 + autoopen];
}


/* -------------------------------   Check   --------------------------------

  Determine if time is a record for level, provided it's not a random game,
  or it's not automatic opening AND marking. If it is a record, display it and
  return true.

*/

bool Times::Check(int level, unsigned time, unsigned spaces, unsigned mines)
{
  if (level < BEGINNER || (isAutoMark && isAutoOpen)) return false;

  Table* table	  = FindTable(level);
  int	 isRecord = table->Enter(time, spaces, mines);
  if (isRecord != -1) TimesWindow(*table, level, isRecord).Run();
  return (isRecord != -1);
}


/* ------------------------------   Display   -------------------------------

  Display the times for level. If both automatics are on, turn them off, since
  there is no time for that option. If level is random, use custom.

*/

void Times::Display(int level)
{
  bool	 mark  = isAutoMark,
	 open  = isAutoOpen;
  Table* table = FindTable((level >= BEGINNER) ? level : CUSTOM);
  if (isAutoMark && isAutoOpen) isAutoMark = isAutoOpen = false;
  TimesWindow(*table, level).Run();
  isAutoMark = mark;
  isAutoOpen = open;
}


/* --------------------------------   Save   --------------------------------

  Write all the times to disk.
  Returns false if the file could not be created, true otherwise.

*/

bool Times::Save(const char* filename)
{
  FILE* file = fopen(filename, "wb");
  if (!file) return false;

  fwrite(Beginner,     sizeof(Beginner),     1, file);
  fwrite(Intermediate, sizeof(Intermediate), 1, file);
  fwrite(Expert,       sizeof(Expert),	     1, file);

  for (CustomTimes* list = head; list->next; list = list->next)
  {
    fwrite(&list->layout, sizeof(list->layout), 1, file);
    fwrite(list->times,   sizeof(list->times),	1, file);
  }

  fclose(file);
  return true;
}


/* --------------------------------   Load   --------------------------------

  Read all the times from disk and create the custom games list and popup menu.
  Returns false if the file could not be opened, true otherwise.

*/

bool Times::Load(const char* filename, CustomPopupRes& custom)
{
  FILE* file = fopen(filename, "rb");
  if (!file) return false;

  fread(Beginner,     sizeof(Beginner),     1, file);
  fread(Intermediate, sizeof(Intermediate), 1, file);
  fread(Expert,       sizeof(Expert),	    1, file);

  CustomTimes* list = head;
  for (;;)
  {
    fread(&list->layout, sizeof(list->layout), 1, file);
    if (feof(file)) break;
    fread(list->times,	 sizeof(list->times),  1, file);
    custom.Insert(list->layout);
    list = list->next = new CustomTimes;
  }
  list->next = NULL;

  fclose(file);
  return true;
}


/* ----------------------------   TimesWindow	----------------------------- */

#define ID_GAMECLEAR	     30001
#define ID_GAMECROSS	     30002
#define ID_AUTOMARK	     30003
#define ID_AUTOOPEN	     30004

#define ID_LEVELBEGINNER     30005
#define ID_LEVELINTERMEDIATE 30006
#define ID_LEVELEXPERT	     30007
#define ID_LEVELCUSTOM	     30008

#define ID_DEVIATION	     30009


DEFINE_RESPONSE_TABLE(TimesWindow, TabWin)
  E_BUTTONUP(ID_OK, cmOK)

  E_BOXCHECKED(  ID_DEVIATION,	       cmTable)
  E_BOXUNCHECKED(ID_DEVIATION,	       cmTable)

  E_BOXCHECKED(  ID_AUTOMARK,	       CheckAutoMark)
  E_BOXCHECKED(  ID_AUTOOPEN,	       CheckAutoOpen)
  E_BOXUNCHECKED(ID_AUTOMARK,	       cmTable)
  E_BOXUNCHECKED(ID_AUTOOPEN,	       cmTable)

  E_BOXCHECKED(  ID_GAMECLEAR,	       CheckGameClear)
  E_BOXCHECKED(  ID_GAMECROSS,	       CheckGameCross)
  E_BOXUNCHECKED(ID_GAMECLEAR,         UnCheckGameClear)
  E_BOXUNCHECKED(ID_GAMECROSS,         UnCheckGameCross)

  E_BOXCHECKED(  ID_LEVELBEGINNER,     CheckLevelBeginner)
  E_BOXCHECKED(  ID_LEVELINTERMEDIATE, CheckLevelIntermediate)
  E_BOXCHECKED(  ID_LEVELEXPERT,       CheckLevelExpert)
  E_BOXCHECKED(  ID_LEVELCUSTOM,       CheckLevelCustom)
  E_BOXUNCHECKED(ID_LEVELBEGINNER,     UnCheckLevelBeginner)
  E_BOXUNCHECKED(ID_LEVELINTERMEDIATE, UnCheckLevelIntermediate)
  E_BOXUNCHECKED(ID_LEVELEXPERT,       UnCheckLevelExpert)
  E_BOXUNCHECKED(ID_LEVELCUSTOM,       UnCheckLevelCustom)

END_RESPONSE_TABLE


#define TIMES_WIDTH  (64*FixedFont->width)
#define TIMES_HEIGHT (27*FixedFont->height)

// Create the times' display and display table.
TimesWindow::TimesWindow(const Table& table, int lev, int highlight) :
  TabWin(NULL, NULL, (ws.GetDeskWidth()  - TIMES_WIDTH)  / 2,
		     (ws.GetDeskHeight() - TIMES_HEIGHT) / 2,
		     (ws.GetDeskWidth()  + TIMES_WIDTH)  / 2 - 1,
		     (ws.GetDeskHeight() + TIMES_HEIGHT) / 2 - 1,
         WA_VISABLE | WA_BORDER | WA_SAVEAREA),
  CustomSave(field->Custom)
{
  const int fw = FixedFont->width,
            fh = FixedFont->height;

  SysFont = FixedFont;
  int oyf = FWIDTH + 2*fh + fh/2,
      oys = oyf + 5*(fh+2) + 4*fh,
      wid = 60*fw;
  for (unsigned j = 0; j < Entries; j++)
  {
    fast[j] = new StaticText(this, "", 2*fw, oyf, wid);
    oyf += fh+2;
    slow[j] = new StaticText(this, "", 2*fw, oys, wid);
    oys += fh+2;
  }
  status = new StaticText(this, "", 2*fw, oyf + fh + fh/2, wid);

  wid = 12*fw; //wid = 15*fw;
  int hyt = 2*(fh+2),
      ox = (TIMES_WIDTH/2 - 15*fw) / 2,
      oy = TIMES_HEIGHT- 5*fh - fh/2, oy1 = oy;
  //game = new RadioButtonContainer(this, ox, oy, ox+wid-1, oy+hyt-1);
  //game->AddButton("C&lear", CLEAR);
  //game->AddButton("C&ross", CROSS);
  //game->SelectButton(Game);
  ox += 3; oy += 3;
  gameclear = new CheckBox(this, ID_GAMECLEAR, "C&lear",
				 ox, oy, wid, (Game == CLEAR));
  oy += fh;
  gamecross = new CheckBox(this, ID_GAMECROSS, "C&ross",
				 ox, oy, wid, (Game == CROSS));

  // ox += 3; oy += hyt-1;
  oy += fh;
  automark = new CheckBox(this,ID_AUTOMARK, "Auto&Mark", ox,oy,wid, isAutoMark);
  oy += fh;
  autoopen = new CheckBox(this,ID_AUTOOPEN, "Auto&Open", ox,oy,wid, isAutoOpen);

  ox += TIMES_WIDTH/2 - 1; oy = oy1;
  hyt = 4*(fh+2);
  //level = new RadioButtonContainer(this, ox, oy, ox+wid-1, oy+hyt-1,
  //                                 RADIO_TYPE_DIP);
  //level->AddButton("&Beginner", BEGINNER);
  //level->AddButton("&Intermediate", INTERMEDIATE);
  //level->AddButton("&Expert", EXPERT);
  //level->AddButton("&Custom", CUSTOM);
  //level->SelectButton((lev < BEGINNER) ? CUSTOM : lev);
  ox += 3; oy += 3;
  levelbeginner = new CheckBox(this, ID_LEVELBEGINNER, "&Beginner",
				     ox, oy, wid, (lev == BEGINNER));
  oy += fh;
  levelintermediate = new CheckBox(this, ID_LEVELINTERMEDIATE, "&Intermediate",
					 ox, oy, wid, (lev == INTERMEDIATE));
  oy += fh;
  levelexpert = new CheckBox(this, ID_LEVELEXPERT, "&Expert",
				   ox, oy, wid, (lev == EXPERT));
  oy += fh;
  levelcustom = new CheckBox(this, ID_LEVELCUSTOM, "&Custom", ox, oy, wid,
				   (lev == CUSTOM || lev < BEGINNER));

  switch (lev)
  {
    case BEGINNER:     lastlevel = levelbeginner;     break;
    case INTERMEDIATE: lastlevel = levelintermediate; break;
    case EXPERT:       lastlevel = levelexpert;       break;
    default:	       lastlevel = levelcustom;
  }

  wid = 10*fw;
  ox = (TIMES_WIDTH - 13*fw) / 2; oy = oy1;
  ox += 3; oy += 3;
  deviation = new CheckBox(this, ID_DEVIATION, "&Deviation", ox, oy, wid);

  wid = StringWidth("OK", FixedFont)*2;
  ox = (TIMES_WIDTH-wid)/2; oy = oy1 + hyt - (SysFontBold->height+4);
  ok = new TextButton(this, ID_OK, ox, oy,
			    ox+wid-1, oy+SysFontBold->height+3, "O&K");

  //AddTabStop(game);
  AddTabStop(gameclear);
  AddTabStop(gamecross);
  AddTabStop(automark);
  AddTabStop(autoopen);
  //AddTabStop(level);
  AddTabStop(levelbeginner);
  AddTabStop(levelintermediate);
  AddTabStop(levelexpert);
  AddTabStop(levelcustom);
  AddTabStop(ok);

  Paint(); RefreshWindow();
  Print(table, highlight);
  SysFont = SysFont1;
}


TimesWindow::~TimesWindow()
{
  for (unsigned j = 0; j < Entries; j++)
  {
    delete fast[j]; delete slow[j];
  }
  delete status;
  //delete game; delete level;
  delete gameclear; delete levelbeginner;
  delete gamecross; delete levelexpert;
  delete automark;  delete levelintermediate;
  delete autoopen;  delete levelcustom;
  delete deviation;
  delete ok;
  field->Custom = CustomSave;
}


/* ----------------------------   PaintWindow	-----------------------------

  Draw the frames and display the titles for the fastest and slowest times,
  and draw the boxes around the selections.

*/

void TimesWindow::PaintWindow(int x1, int y1, int x2, int y2)
{
  Window::PaintWindow(x1, y1, x2, y2);

  ViewBuffer* buf = GetSubBuffer(x1, y1, x2, y2);
  const int   fw  = FixedFont->width,
	      fh  = FixedFont->height;

  int oy = FWIDTH + fh/2,
      hyt = 2*fh + 5*(fh+2) + fh/2;
  Frame(buf, fw, oy, TIMES_WIDTH-fw-1, oy+hyt-1);
  oy += hyt + 3*fh;
  InvFrame(buf, fw, oy, TIMES_WIDTH-fw-1, oy+hyt-1);
  int len = StringWidth("FASTEST TIMES", FixedFont);
  WriteText(buf, (TIMES_WIDTH-len)/2, FWIDTH+fh,
                 HighLightColour, FixedFont, "FASTEST TIMES");
  len = StringWidth("SLOWEST TIMES", FixedFont);
  WriteText(buf, (TIMES_WIDTH-len)/2, oy+hyt-fh-fh/2,
                 HighLightColour, FixedFont, "SLOWEST TIMES");

  oy = TIMES_HEIGHT - 5*fh - fh/2 - 1;
  hyt = 4*(fh+2) + 1;
  int wid = 15*fw,
      ox = (TIMES_WIDTH/2 - wid) / 2 - 1,
      ey = oy + hyt-2;
  HDip(buf, ox, ox+wid-2, oy);
  VDip(buf, ox, oy+1, ey);
  HDip(buf, ox, ox+wid-2, ey);
  VDip(buf, ox+wid-2, oy+1, ey);

  ox += 3 + TIMES_WIDTH/2 - 1;
  HDip(buf, ox, ox+wid-2, oy);
  VDip(buf, ox, oy+1, ey);
  HDip(buf, ox, ox+wid-2, ey);
  VDip(buf, ox+wid-2, oy+1, ey);

  wid = 13*fw;
  ox = (TIMES_WIDTH - wid) / 2 - 1;
  ey = oy + (fh+2)+1 + 4;
  HDip(buf, ox, ox+wid-2, oy);
  VDip(buf, ox, oy+1, ey);
  HDip(buf, ox, ox+wid-2, ey);
  VDip(buf, ox+wid-2, oy+1, ey);

  DeleteBuffer(buf);
}


/* -------------------------------   Print   --------------------------------

  Update the window with the values from table. If highlight is not -1, display
  the appropriate time using the highlight colour.

*/

void TimesWindow::Print(const Table& table, int highlight)
{
  //bool clear = (game->GetSelectedButton() == CLEAR);
  bool clear = gameclear->isChecked();

  bool	 dev = deviation->isChecked();
  double avg = (table.games) ? (double)table.timetotal / table.games : 0;

  for (unsigned j = 0; j < Entries; j++)
  {
    fast[j]->SetText(table.fastest[j].str(clear, dev, avg));
    slow[j]->SetText(table.slowest[Entries-1 - j].str(clear, dev, avg));
  }
  status->SetText(table.str(clear));

  if (highlight >= 0 && highlight < (int)Entries*2)
  {
    StaticText* which = (highlight < (int)Entries) ? fast[highlight] :
				       slow[Entries-1 - (highlight - Entries)];
    WriteText(which->GetBuffer(), 0, 0, HighLightColour, FixedFont,
	      which->GetText());
    which->RefreshWindow();
  }
}


void TimesWindow::Run()
{
  SysFont = FixedFont;
  SetFocus(ok);
  CatchMouse();
  CatchKeys();
  ws.RunEvents();
  ws.ResetEvents();
  ReleaseMouse();
  ReleaseKeys();
  SysFont = SysFont1;
}


// Since there's no automatic opening AND marking, uncheck the other one if
// it's checked.
void TimesWindow::CheckAutoMark()
{
  if (autoopen->isChecked()) autoopen->UnCheck();
  cmTable();
}

void TimesWindow::CheckAutoOpen()
{
  if (automark->isChecked()) automark->UnCheck();
  cmTable();
}


/* ----------------------   Simulated radio buttons   -----------------------

  Simulate radio buttons with checkboxes - when checking, uncheck the one
  currently checked. When unchecking, re-check it.

*/

void TimesWindow::CheckGameClear()
{
  if (gamecross->isChecked()) gamecross->UnCheck();
  cmTable();
}

void TimesWindow::CheckGameCross()
{
  if (gameclear->isChecked()) gameclear->UnCheck();
  cmTable();
}

void TimesWindow::UnCheckGameClear()
{
  if (!gamecross->isChecked()) gameclear->Check();
}

void TimesWindow::UnCheckGameCross()
{
  if (!gameclear->isChecked()) gamecross->Check();
}

void TimesWindow::CheckLevelBeginner()
{
  if (lastlevel == levelbeginner) return;
  lastlevel->UnCheck();
  lastlevel = levelbeginner;
  cmTable();
}

void TimesWindow::CheckLevelIntermediate()
{
  if (lastlevel == levelintermediate) return;
  lastlevel->UnCheck();
  lastlevel = levelintermediate;
  cmTable();
}

void TimesWindow::CheckLevelExpert()
{
  if (lastlevel == levelexpert) return;
  lastlevel->UnCheck();
  lastlevel = levelexpert;
  cmTable();
}


// If there's no custom games then prevent this from being selected,
// otherwise pop-up the menu underneath the "Custom".
void TimesWindow::CheckLevelCustom()
{
  if (field->CustomPopup.number == 0)
  {
    levelcustom->UnCheck();
    return;
  }

  if (lastlevel != levelcustom) lastlevel->UnCheck();
  CheckBox* curlevel = lastlevel;
  lastlevel = levelcustom;

  int ox = absx + TIMES_WIDTH*3/4 - 6*FixedFont->width + 3 + 20,
      oy = absy + TIMES_HEIGHT - 5*FixedFont->height - FixedFont->height/2 +
                  4*(FixedFont->height+2) - 2;
  if (CustomPopupMenu(NULL, field->CustomPopup, ox, oy).Run())
    cmTable();
  else
    if (curlevel != levelcustom) curlevel->Check();

  CatchKeys();                          // It seems PopupMenu releases keys
}


void TimesWindow::UnCheckLevelBeginner()
{
  if (lastlevel == levelbeginner) levelbeginner->Check();
}

void TimesWindow::UnCheckLevelIntermediate()
{
  if (lastlevel == levelintermediate) levelintermediate->Check();
}

void TimesWindow::UnCheckLevelExpert()
{
  if (lastlevel == levelexpert) levelexpert->Check();
}

void TimesWindow::UnCheckLevelCustom()
{
  if (field->CustomPopup.number == 0) return;
  if (lastlevel == levelcustom) levelcustom->Check();
}


/* ------------------------------   CmTable   -------------------------------

  Find the table corresponding to the selections and display it.

*/

void TimesWindow::cmTable()
{
  int level, game;
       if (levelbeginner->isChecked())	   level = BEGINNER;
  else if (levelintermediate->isChecked()) level = INTERMEDIATE;
  else if (levelexpert->isChecked())	   level = EXPERT;
  else					   level = CUSTOM;
  game = (gameclear->isChecked()) ? CLEAR : CROSS;

  Table* table = field->times.FindTable(level,//->GetSelectedButton(),
					game,//->GetSelectedButton(),
					automark->isChecked(),
					autoopen->isChecked());
  Print(*table);
}


/* ----------------------------   EnterNameDlg	 ---------------------------- */

#define NAME_WIDTH  (50*SysFont1->width)
#define NAME_HEIGHT (12*SysFont1->height)

#define NAME_ID 30000

DEFINE_RESPONSE_TABLE(EnterNameDlg, Window)
  E_KEYCODE(0x0d, 0x1c, KeyEnter)
  E_BUTTONUP(ID_OK,	cmOK)
END_RESPONSE_TABLE

char EnterNameDlg::lastname[NameLen] = { 0 };


// Create the enter name dialog box, where message indicates whether it was a
// fast or slow time. The name is entered using the fixed font.
EnterNameDlg::EnterNameDlg(const char* message) :
  Window(NULL, "Congratulations", (ws.GetDeskWidth() -  NAME_WIDTH)  / 2,
				  (ws.GetDeskHeight() - NAME_HEIGHT) / 2,
				  (ws.GetDeskWidth() +	NAME_WIDTH)  / 2 - 1,
				  (ws.GetDeskHeight() + NAME_HEIGHT) / 2 - 1,
         WA_VISABLE | WA_CAPTION | WA_BORDER | WA_SAVEAREA)
{
  int len = StringWidth(message, SysFont);
  msg = new StaticText(this, message,
                       (NAME_WIDTH-len)/2, SysFont->height*3, len);
  len = NameLen*FixedFont->width;
  SysFont = FixedFont;
  name = new EditText(this, NAME_ID, "", NameLen,
                      (NAME_WIDTH-len)/2, SysFont1->height*6, len);
  SysFont = SysFont1;
  len = StringWidth("OK", SysFont)*2;
  int ox = (NAME_WIDTH-len)/2, oy = cy2-SysFont->height*2-4;
  ok = new TextButton(this, ID_OK, ox,oy, ox+len-1, oy+SysFont->height+3, "OK");
  SetFocus(name);
  name->SetText(lastname);
}


EnterNameDlg::~EnterNameDlg()
{
  delete msg;
  delete name;
  delete ok;
}


// Write the prompt onto the window.
void EnterNameDlg::PaintWindow(int x1, int y1, int x2, int y2)
{
  Window::PaintWindow(x1, y1, x2, y2);

  ViewBuffer* buf = GetSubBuffer(x1, y1, x2, y2);
  WriteText(buf, (NAME_WIDTH-NameLen*FixedFont->width)/2, SysFont->height*5,
                 HighLightColour, SysFontItallic, "Please enter your name:");
  DeleteBuffer(buf);
}


char* EnterNameDlg::Run()
{
  Paint();
  SysFont = FixedFont;
  name->Paint();
  RefreshWindow();
  CatchMouse();
  CatchKeys();
  ws.RunEvents();
  ws.ResetEvents();
  ReleaseMouse();
  ReleaseKeys();
  SysFont = SysFont1;
  strcpy(lastname, name->GetText());
  return (*lastname == 0) ? "Anonymous" : lastname;
}
